# Simple Memory Match Game in Python 3 in Simcenter Amesim

import tkinter as tk
from random import shuffle
import os
from pathlib import Path
from random import randrange
from do_image import find_all_image, resize_images
from tkinter import messagebox
from tkinter import filedialog as fd

script_path = Path( __file__ ).parent.absolute()
os.chdir(script_path)

class Game:
    """Whole game run in this class"""
    def __init__(self):
        self.levels = ('6', '10', '15', '20')
        self.default_img_dir_name = 'Simcenter Amesim Libraries'
        self.img_dir_name = self.default_img_dir_name

    def new_game(self, pairs=6):
        """Creating window of game and selection of images."""
        self.number_of_pairs_str = str(pairs)
        self.card_click_history = []
        self.number_of_moves = 0

        # If exist last game, delete old game
        try: self.game_destroy()
        except: pass

        self.do_menu()
        self.create_sheet_and_frames()
        self.set_images()
        self.last_game_pairs = pairs

    def create_sheet_and_frames(self):
        # Creating a sheet of cards. Rows and columns.
        if self.number_of_pairs_str == '6':
            self.number_of_row = 3
            self.number_of_column = 4
        elif self.number_of_pairs_str == '10':
            self.number_of_row = 4
            self.number_of_column = 5
        elif self.number_of_pairs_str == '15':
            self.number_of_row = 5
            self.number_of_column = 6
        elif self.number_of_pairs_str == '20':
            self.number_of_row = 5
            self.number_of_column = 8
        # Do main frame
        self.frame = tk.Frame(root)
        self.frame.pack()
        root.title(f'Simcenter Amesim Memory Match Game - {self.number_of_pairs_str} pairs')

        # Do bottom frame (display the number of moves and the record)
        self.frame_player = tk.Frame(root)
        self.label_moves = tk.Label(self.frame_player, text=f'Moves: {self.number_of_moves}', font=('calibre', 14, 'normal'))
        self.label_moves.pack(side=tk.TOP)
        self.frame_player.pack()

    def do_menu(self):
        # Making menu and add to the 'root'
        self.menu = tk.Menu()
        self.filemenu= tk.Menu(self.menu, tearoff=0)
        self.filemenu.add_cascade(label='Change images to play with', command=self.change_images_directory)
        self.gamemenu = tk.Menu(self.menu, tearoff=0)
        self.gamemenu.add_command(label='6 pars', command=lambda: self.new_game(6))
        self.gamemenu.add_command(label='10 pars', command=lambda: self.new_game(10))
        self.gamemenu.add_command(label='15 pars', command=lambda: self.new_game(15))
        self.gamemenu.add_command(label='20 pars', command=lambda: self.new_game(20))

        self.menu.add_cascade(label='New game', menu=self.gamemenu)
        self.menu.add_cascade(label='Images', menu=self.filemenu)
        root.config(menu=self.menu)

    def set_images(self):
        self.image_list = []
        self.all_images = find_all_image(rf'images\{self.img_dir_name}')
        # Delete default image form list
        try:
            self.all_images.remove('0.png')
        except ValueError:
            messagebox.showerror('warning', f'There is no default photo: "0.png" in the selected catalog: "{self.img_dir_name}"')
            self.change_images_directory()

        if len(self.all_images) < int(self.number_of_pairs_str):
            messagebox.showwarning('warning', f'Not enough pictures in the selected catalog: "{self.img_dir_name}"\nYou need {self.number_of_pairs_str} pictures.\n'
                                              f'Add more pictures or change the directory.',)
        else:
            # Selecting the images.
            for i in range(int(self.number_of_pairs_str)):
                self.image_list.append(self.all_images.pop(randrange(len(self.all_images))))
            # All cards in list * 2
            self.image_list *= 2

            # Mixed images in list
            shuffle(self.image_list)

            # Making a list with [] * 'self.number_of_row'
            self.fields = [[] for i in range(self.number_of_row)]

            # Making objects class 'Field' and add to the list 'self.fields'
            for row in range(self.number_of_row):
                for column in range(self.number_of_column):
                    self.fields[row].append(Field(self.image_list.pop(0), row, column, self))

    def change_images_directory(self):
        # Do directory list
        self.dir_list = next(os.walk(r'.\images'))[1]

        self.window_change_dir_img = tk.Toplevel(root)
        self.window_change_dir_img.title('Select images to play with:')
        self.label_change_dir_img = tk.Label(self.window_change_dir_img, font=('calibre', 14, 'normal'), padx=20, pady=15,
                                             text='Select images to play with:')
        self.label_change_dir_img.pack()

        self.var = tk.StringVar(value='0')
        for dir in self.dir_list:
            tk.Radiobutton(self.window_change_dir_img, variable=self.var, text=dir, value=dir, font=('calibre', 12, 'normal')).pack()

        self.button_change_dir_img_OK = tk.Button(self.window_change_dir_img, text='OK', padx=10, pady=5,
                                                  command=lambda: self.set_img_dir_name(self.var.get()))
        self.button_change_dir_img_OK.pack()

    def set_img_dir_name(self, dir):
        if dir != '0':
            self.img_dir_name = dir
        else:
            self.img_dir_name = self.default_img_dir_name

        self.window_change_dir_img.destroy()
        self.new_game(self.last_game_pairs)

    def check_pair(self, click_card):
        """Checking for a match. Object of field is argument."""
        # Add field to the 'self.card_click_history'
        if len(self.card_click_history) == 0:
            self.card_click_history.append(click_card)

        # If the second card is exposed
        elif len(self.card_click_history) == 1:
            self.card_click_history.append(click_card)
            self.number_of_moves += 1
            # Display number of move
            self.label_moves.config(text=f'Moves: {self.number_of_moves}')

            # If the cards match then turn off the activity of the fields.
            if self.card_click_history[0].name == self.card_click_history[1].name:
                self.card_click_history[0].set_disable_field()
                self.card_click_history[1].set_disable_field()
            # If the cards don't match then wait and load the default image.
            elif self.card_click_history[0].name != self.card_click_history[1].name:
                self.frame.after(200, self.card_click_history[0].set_default_image())
                self.frame.after(200, self.card_click_history[1].set_default_image())

            # Delete filds of moves
            self.card_click_history = []

    def end_game(self):
        """Check that at least one field is active"""
        for row in self.fields:
            for field in row:
                if field.field_activity == True:
                    return False

        return True

    def game_destroy(self):
        """Destroy all frames"""
        self.frame.destroy()
        self.frame_player.destroy()

class Field:
    """The class create a buttons of field and set button activity"""
    def __init__(self, name, row, column, game):
        self.frame = game.frame
        self.name = name
        self.image = tk.PhotoImage(file=rf'images\{game.img_dir_name}\{self.name}')
        self.default_image = tk.PhotoImage(file=rf'images\{game.img_dir_name}\0.png')
        self.row = row
        self.column = column
        self.field_activity = True
        self.field_button = tk.Button(self.frame, image=self.default_image, command=self.click).grid(row=self.row, column=self.column)

    def set_disable_field(self):
        # Set activity on False and set own image.
        self.field_button = tk.Button(self.frame, image=self.image, state=tk.DISABLED).grid(row=self.row, column=self.column)
        self.field_activity = False

    def set_default_image(self):
        # Set activity on True and set default image.
        self.field_button = tk.Button(self.frame, image=self.default_image, command=self.click).grid(row=self.row, column=self.column)
        self.field_activity = True

    def click(self):
        # If field is activ, change activity to False and display image. Calls the 'check_pair' method.
        if self.field_activity == True:
            self.field_button = tk.Button(self.frame, image=self.image).grid(row=self.row, column=self.column)
            self.field_activity = False
            self.frame.update()
            game.check_pair(self)

if __name__ == "__main__":
    root = tk.Tk()
    # Block the resize of window
    root.resizable(False, False)
    # Set minimal size
    root.minsize(300, 200)

    game = Game()
    game.new_game()

    root.mainloop()